/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include <dawn/dawn.h>
#include "../card.h"
#include "../../util/rand.h"
#include "../../display/spritebatch.h"
#include "../../display/texture.h"
#include "../../display/tileset.h"
#include "../../display/shader.h"
#include "../../file/asset.h"

/**
 * Initializes a Teax Hold'em Game for the first time.
 * 
 * @param game Game to initialize.
 */
void holdemGameInit(holdemgame_t *game);

/**
 * Initializes a Texas Hold'em Round for the first time. Does not affect the
 * global game state.
 * 
 * @param game Game's round you want to initialize.
 */
void holdemRoundInit(holdemgame_t *game);

/**
 * Deals a card to the given player, does all the follow up.
 * 
 * @param game Game who's deck to deal from.
 * @param player Player to deal into.
 */
void holdemDeal(holdemgame_t *game, holdemplayer_t *player);

/**
 * Deal cards to all players.
 * 
 * @param game Game and players to deal around.
 * @param count Count of cards to deal to each player.
 */
void holdemDealAll(holdemgame_t *game, uint8_t count);

/**
 * Draw the flop, turn or river
 * 
 * @param game Game to flop/turn/river.
 */
void holdemFlop(holdemgame_t *game);


/**
 * Initialize the Texas Hold'em Renderer.
 * 
 * @param render Renderer to init.
 */
void holdemRenderInit(holdemrender_t *render);

/**
 * Renders a card.
 * 
 * @param render Renderer to render against
 * @param card Card to render.
 */
void holdemRenderCard(holdemrender_t *render, card_t card, bool faceUp, float x, float y, float z);

/** 
 * Render an entire holdem game.
 */
void holdemRender(holdemrender_t *render, holdemgame_t *game);

/**
 * Takes the given bet from a player
 * 
 * @param game Game to add the bet to.
 * @param player Player to take the bet from.
 * @param amount Amount to try and take.
 * @return The real amount that was taken, chips considered.
 */
uint32_t holdemBet(holdemgame_t *game, holdemplayer_t *player, uint32_t amount);