/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "game.h"

game_t GAME_STATE;

primitive_t *prim;
framebuffer_t *fbo;
texture_t *text;

bool gameInit() {
  // Init the game
  GAME_STATE.name = GAME_NAME;

  // Init the renderer.
  gameTimeInit();
  renderInit();
  inputInit();
  worldInit();

  prim = cubeCreate(1, 1, 1);
  text = assetTextureLoad("bruh.png");

  fbo = frameBufferCreate(128, 128);

  // Load the world shader.
  GAME_STATE.shaderWorld = assetShaderLoad(
    "shaders/textured.vert", "shaders/textured.frag"
  );

  // Init the input manger.
  return true;
}

bool gameUpdate(float platformDelta) {
  gameTimeUpdate(platformDelta);
  renderFrameStart();
  inputUpdate();

  // Set up the camera and shader.
  shaderUse(GAME_STATE.shaderWorld);
  frameBufferUse(fbo, true);
  cameraLookAt(&GAME_STATE.cameraWorld, 2, 2, 2, 0, 0, 0);
  cameraPerspective(&GAME_STATE.cameraWorld, 45.0f,
    ((float)fbo->texture->width) / ((float)fbo->texture->height),
    0.5f, 500.0f
  );
  shaderUseCamera(GAME_STATE.shaderWorld, &GAME_STATE.cameraWorld);
  shaderUsePosition(GAME_STATE.shaderWorld, 0, 0, 0, TIME_STATE.current, TIME_STATE.current, 0);
  shaderUseTexture(GAME_STATE.shaderWorld, text);
  primitiveDraw(prim, 0, prim->indiceCount);

  frameBufferUse(NULL, true);
  shaderUseTexture(GAME_STATE.shaderWorld, fbo->texture);
  cameraPerspective(&GAME_STATE.cameraWorld, 45.0f,
    ((float)RENDER_STATE.width) / ((float)RENDER_STATE.height),
    0.5f, 500.0f
  );
  shaderUseCamera(GAME_STATE.shaderWorld, &GAME_STATE.cameraWorld);
  primitiveDraw(prim, 0, prim->indiceCount);

  if(inputIsPressed(INPUT_NULL)) return false;
  return true;
}

void gameDispose() {
  shaderDispose(GAME_STATE.shaderWorld);
  worldDispose();
  inputDispose();
  renderDispose();
}